﻿Type=Class
Version=2.20
@EndOfDesignText@
'Class module
Sub Class_Globals
	Type ActionItem (Text As String, Image As Bitmap, Value As Object)
	Private Pnl As Panel
	Private Canvas As Canvas
	Private Actions As List
	Public Visible As Boolean
	Private Title As String
	Private InAnimation As Animation
	Private OutAnimation As Animation
	Private Selected As Int
	Private ActivityName As String
	Private ItemHeight As Int : ItemHeight = 45dip
	Private TextSize As Float : TextSize = 16 'NOT dip.
	Private TextLeftPos As Int : TextLeftPos = 40dip
	Private TextColor As Int : TextColor = Colors.LightGray
	Private TopPadding As Int : TopPadding = 5dip
	Private ImageWidth As Int : ImageWidth = 16dip
	Private ImageHeight As Int : ImageHeight = 16dip
	Private ImageLeftPos As Int : ImageLeftPos = 10dip
	Private ImageTopPos As Int : ImageTopPos = (ItemHeight - ImageHeight) / 2
	Private SelectionColor As Int : SelectionColor = 0xFFFF8000
	Private BackgroundImage As Object
End Sub

'Initializes the object. You can add parameters to this method if needed.
Sub Initialize (Width As Int, Title1 As String, ActivityName1 As String)
	BackgroundImage = CreateNinePatchDrawable("bg")
	Actions.Initialize
	Dim bmp As Bitmap
	bmp.InitializeMutable(Width, 100%y)
	Canvas.Initialize2(bmp)
	Pnl.Initialize("ActionList")
	Dim bd As BitmapDrawable
	bd.Initialize(bmp)
	bd.Gravity = Bit.Or(Gravity.LEFT, Gravity.TOP)
	Pnl.Background = bd
	Title = Title1
	InAnimation.InitializeAlpha("", 0, 1)
	InAnimation.Duration = 300
	OutAnimation.InitializeAlpha("", 1, 0)
	OutAnimation.Duration = 200
	Selected = -1
	ActivityName = ActivityName1
End Sub


Sub AddActionItem(Text As String, Image As Bitmap, ReturnValue As Object)
	Dim item As ActionItem
	item.Initialize
	item.Text = Text
	item.Image = Image
	item.Value = ReturnValue
	Actions.Add(item)
End Sub

Sub Show (Activity As Activity, Left As Int, Top As Int)
	If Visible Then 
		InAnimation.Stop(Pnl)
		Pnl.RemoveView
	End If
	Dim h As Int
	h = (Actions.Size + 1) * ItemHeight + TopPadding
	Activity.AddView(Pnl, Left, Top, Canvas.Bitmap.Width, h)
	If Visible = False Then
		InAnimation.Start(Pnl)
	End If
	Dim c As Canvas
	c = Canvas
	Dim r As Rect
	r.Initialize(0, 0, Pnl.Width, Pnl.Height)
	c.DrawRect(r, Colors.Transparent, True, 0)
	c.DrawDrawable(BackgroundImage, r)
	If Title.Length > 0 Then
		c.DrawText(Title, Pnl.Width / 2, TopPadding + ItemHeight / 2, Typeface.DEFAULT_BOLD, _
			 TextSize, TextColor, "CENTER")
	End If
	
	'Draw the items
	For i = 0 To Actions.Size - 1
		drawItem(i)
	Next
	Pnl.Invalidate
	Visible = True
End Sub

Sub Hide
	If Visible = False Then Return
	Visible = False
	OutAnimation.Start(Pnl)
	Pnl.RemoveView
End Sub
Private Sub ActionList_Touch(Action As Int, X As Float, Y As Float)
	If Action <> 1 Then '1 is the value of Activity.ACTION_UP (not accessible from the code module)
		Dim currentItem As Int
		currentItem = Floor((Y - TopPadding - ItemHeight) / ItemHeight)
		If currentItem = Selected Then Return
		changeSelection(currentItem)
	Else If Selected >= 0 Then
		Hide
		Dim item As ActionItem
		item = Actions.Get(Selected)
		CallSub2(ActivityName, "ActionList_Result", item.Value) 
		Selected = -1
	End If
End Sub
Private Sub drawItem(Index As Int)
	Dim item As ActionItem
	item = Actions.Get(Index)
	If item.Image.IsInitialized Then
		Dim imageRect As Rect
		imageRect.Initialize(ImageLeftPos, 0, ImageLeftPos + ImageWidth, 0)
		imageRect.Top = (Index + 1) * ItemHeight + TopPadding + ImageTopPos
		imageRect.Bottom = imageRect.Top + ImageHeight
		Canvas.DrawBitmap(item.Image, Null, imageRect)
	End If
	
	Canvas.DrawText(item.Text, TextLeftPos, _
		(Index + 1 + 0.5) * ItemHeight + TopPadding + Canvas.MeasureStringHeight(item.Text, Typeface.DEFAULT, TextSize) /2 _ 
			, Typeface.DEFAULT, TextSize, TextColor, "LEFT")
End Sub
Private Sub changeSelection(NewSelection As Int)
	Dim r As Rect
	Dim top As Int
	'delete the old selection
	If Selected >= 0 Then
		Dim p As Path
		Dim y As Int
		y = (Selected + 1) * ItemHeight + TopPadding
		p.Initialize(0,  y)
		p.LineTo(0, y + ItemHeight)
		p.LineTo(Pnl.Width, y + ItemHeight)
		p.LineTo(Pnl.Width, y)
		Canvas.ClipPath(p)
		r.Initialize(0, 0, Pnl.Width, Pnl.Height)
		Canvas.DrawRect(r, Colors.Transparent, True, 0)
		Canvas.DrawDrawable(BackgroundImage, r) 
		drawItem(Selected)
		Canvas.RemoveClip
	End If
	If NewSelection >= 0 AND NewSelection < Actions.Size Then
		top = (NewSelection + 1) * ItemHeight + TopPadding + 3dip
		r.Initialize(3dip, top, Pnl.Width - 3dip, top + ItemHeight - 6dip)
		Canvas.DrawRect(r, SelectionColor, True, 0)
		drawItem(NewSelection)
		Selected = NewSelection
	Else
		Selected = -1
	End If
	Pnl.Invalidate
End Sub
Private Sub CreateNinePatchDrawable(ImageName As String) As Object
    Dim r As Reflector
    Dim package As String
    Dim id As Int
    package = r.GetStaticField("anywheresoftware.b4a.BA", "packageName")
    id = r.GetStaticField(package & ".R$drawable", ImageName)
    r.Target = r.GetContext
    r.Target = r.RunMethod("getResources")
    Return r.RunMethod2("getDrawable", id, "java.lang.int")
End Sub